<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Uper for Elementor
 * Customizable scroll to top button for Elementor editor
 * Exclusively on https://1.envato.market/uper-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.4
 * @copyright       (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\UperElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Plugin;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\UperElementor\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Uper - Custom Elementor Widget.
 **/
class uper_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-uper-elementor-admin', UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-uper-elementor', UnityPlugin::get_url() . 'css/uper-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
	    wp_register_script( 'mdp-uper-elementor', UnityPlugin::get_url() . 'js/uper-elementor' . UnityPlugin::get_suffix() . '.js', [ 'elementor-frontend' ], UnityPlugin::get_version(), true );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-uper-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Uper', 'uper-elementor' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-uper-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Uper' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-uper-elementor', 'mdp-uper-elementor-admin', 'elementor-icons-fa-solid' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-uper-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Button Section. */
        $this->section_content_button();

        /** Content -> Scrolling Section. */
        $this->section_content_scrolling();

        /** Content -> Button Content Section. */
        $this->section_content_button_content();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Button. */
        $this->section_style_button();

        /** Style -> Section Style Scroll Indicator. */
        $this->section_style_scroll_indicator();

    }

    /**
     * Add widget controls: Content -> Button Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_button() {

        $this->start_controls_section( 'section_content_button', [
            'label' => esc_html__( 'Button', 'uper-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'alignment',
            [
                'label' => esc_html__( 'Alignment', 'uper-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'uper-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'uper-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'uper-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'condition' => [
                        'fixed!' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-box' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'fixed',
            [
                'label' => esc_html__( 'Fixed', 'uper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'uper-elementor' ),
                'label_off' => esc_html__( 'No', 'uper-elementor' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'fixed_button_position',
            [
                'label' => esc_html__( 'Fixed position', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom-right',
                'options' => [
                    'top-left'  => esc_html__( 'Top left', 'uper-elementor' ),
                    'top-center' => esc_html__( 'Top center', 'uper-elementor' ),
                    'top-right' => esc_html__( 'Top right', 'uper-elementor' ),
                    'left-center' => esc_html__( 'Left center', 'uper-elementor' ),
                    'right-center' => esc_html__( 'Right center', 'uper-elementor' ),
                    'bottom-left' => esc_html__( 'Bottom left', 'uper-elementor' ),
                    'bottom-center' => esc_html__( 'Bottom center', 'uper-elementor' ),
                    'bottom-right' => esc_html__( 'Bottom right', 'uper-elementor' ),
                ],
                'condition' => [
                        'fixed' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'scroll_indicator',
            [
                'label' => esc_html__( 'Scroll indicator', 'uper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'uper-elementor' ),
                'label_off' => esc_html__( 'Hide', 'uper-elementor' ),
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'fixed' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'scroll_indicator_type',
            [
                'label' => esc_html__( 'Scroll indicator type', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'circle',
                'options' => [
                    'circle' => esc_html__( 'Circle', 'uper-elementor' ),
                    'background' => esc_html__( 'Background', 'uper-elementor' ),
                ],
                'condition' => [
                    'fixed' => 'yes',
                    'scroll_indicator' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'auto_hide',
            [
                'label' => esc_html__( 'Auto hide', 'uper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'uper-elementor' ),
                'label_off' => esc_html__( 'Hide', 'uper-elementor' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                        'fixed' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'auto_hide_after',
            [
                'label' => esc_html__( 'Auto hide after', 'uper-elementor' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 1000,
                'step' => 1,
                'default' => 10,
                'condition' => [
                        'auto_hide' => 'yes',
                        'fixed' => 'yes'
                ],
                'description' => wp_kses_post( 'Specify after how many seconds button will disappear' ),
            ]
        );

        $this->add_control(
            'start_displaying_button_position',
            [
                'label' => esc_html__( 'Start displaying button position', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'pixels',
                'label_block' => true,
                'options' => [
                    'pixels'  => esc_html__( 'Pixels', 'uper-elementor' ),
                    'percentage' => esc_html__( 'Percentage', 'uper-elementor' ),
                    'custom-element' => esc_html__( 'Custom element', 'uper-elementor' ),
                    'always-show' => esc_html__( 'Always show', 'uper-elementor' )
                ],
                'condition' => [
                    'fixed' => 'yes'
                ]
            ]
        );


        $this->add_control(
            'start_displaying_button_position_pixels',
            [
                'label' => esc_html__( 'Pixels', 'uper-elementor' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 1000,
                'step' => 1,
                'default' => 100,
                'condition' => [
                    'start_displaying_button_position' => 'pixels',
                    'fixed' => 'yes'
                ]
            ]
        );


        $this->add_control(
            'start_displaying_button_position_percentage',
            [
                'label' => esc_html__( 'Percentage', 'uper-elementor' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 100,
                'step' => 1,
                'default' => 10,
                'condition' => [
                    'start_displaying_button_position' => 'percentage',
                    'fixed' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'start_displaying_button_position_element',
            [
                'label' => esc_html__( 'Element ID', 'uper-elementor' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type element id here', 'uper-elementor' ),
                'separator' => 'after',
                'condition' => [
                    'start_displaying_button_position' => 'custom-element',
                    'fixed' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'button_appearance_animation',
            [
                'label' => esc_html__( 'Button appearance animation', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'label_block' => true,
                'default' => 'none',
                'options' => [
                    'grow' => esc_html__( 'Grow', 'uper-elementor' ),
                    'shrink' => esc_html__( 'Shrink', 'uper-elementor' ),
                    'slide-up' => esc_html__( 'Slide up', 'uper-elementor' ),
                    'slide-down' => esc_html__( 'Slide down', 'uper-elementor' ),
                    'fade' => esc_html__( 'Fade', 'uper-elementor' ),
                    'none' => esc_html__( 'None', 'uper-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'button_appearance_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'uper-elementor' ),
                    'ease-in' => esc_html__( 'Ease-in', 'uper-elementor' ),
                    'ease-out' => esc_html__( 'Ease-out', 'uper-elementor' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'uper-elementor' ),
                    'linear' => esc_html__( 'Linear', 'uper-elementor' ),
                ],
                'condition' => [
                    'button_appearance_animation!' => 'none',
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-uper-elementor-button' => 'animation-timing-function: {{VALUE}} !important'
                ],
            ]
        );

        $this->add_control(
            'button_appearance_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-uper-elementor-button' => 'animation-duration: {{SIZE}}{{UNIT}} !important'
                ],
                'condition' => [
                    'button_appearance_animation!' => 'none',
                ],
            ]
        );

        $this->add_control(
            'button_appearance_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-uper-elementor-button' => 'animation-delay: {{SIZE}}{{UNIT}} !important'
                ],
                'condition' => [
                    'button_appearance_animation!' => 'none',
                ],
            ]
        );

        $this->add_control(
            'button_hover_animation',
            [
                'label' => esc_html__( 'Button hover animation', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'label_block' => true,
                'default' => 'none',
                'options' => [
                    'grow-hover' => esc_html__( 'Grow', 'uper-elementor' ),
                    'shrink-hover' => esc_html__( 'Shrink', 'uper-elementor' ),
                    'slide-up-hover' => esc_html__( 'Slide up', 'uper-elementor' ),
                    'slide-down-hover' => esc_html__( 'Slide down', 'uper-elementor' ),
                    'swing-hover' => esc_html__( 'Swing', 'uper-elementor' ),
                    'none' => esc_html__( 'None', 'uper-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'button_hover_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'uper-elementor' ),
                    'ease-in' => esc_html__( 'Ease-in', 'uper-elementor' ),
                    'ease-out' => esc_html__( 'Ease-out', 'uper-elementor' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'uper-elementor' ),
                    'linear' => esc_html__( 'Linear', 'uper-elementor' ),
                ],
                'condition' => [
                    'button_hover_animation!' => 'none',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button:hover' => 'animation-timing-function: {{VALUE}} !important'
                ],
            ]
        );

        $this->add_control(
            'button_hover_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button:hover' => 'animation-duration: {{SIZE}}{{UNIT}} !important'
                ],
                'condition' => [
                    'button_hover_animation!' => 'none',
                ],
            ]
        );

        $this->add_control(
            'button_hover_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button:hover' => 'animation-delay: {{SIZE}}{{UNIT}} !important'
                ],
                'condition' => [
                    'button_hover_animation!' => 'none',
                ],
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Scrolling Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_scrolling()
    {

        $this->start_controls_section('section_content_scrolling', [
            'label' => esc_html__('Scrolling', 'uper-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'scroll_speed',
            [
                'label' => esc_html__( 'Scroll speed', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
            ]
        );

        $this->add_control(
            'scroll_easing',
            [
                'label' => esc_html__( 'Scroll easing', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'smooth',
                'options' => [
                    'start-slowdown' => esc_html__( 'Slowdown on start', 'uper-elementor' ),
                    'end-slow-down' => esc_html__( 'Slowdown in the end', 'uper-elementor' ),
                    'smooth' => esc_html__( 'Smoothly on start and end', 'uper-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'scroll_to',
            [
                'label' => esc_html__( 'Scroll to', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'top',
                'options' => [
                    'top' => esc_html__( 'Top', 'uper-elementor' ),
                    'anchor' => esc_html__( 'Anchor', 'uper-elementor' ),
                ],
            ]
        );


        $this->add_control(
            'anchor_id',
            [
                'label' => esc_html__( 'Element Id', 'uper-elementor' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type element id here', 'uper-elementor' ),
                'condition' => [
                       'scroll_to' => 'anchor'
                ]
            ]
        );


        $this->end_controls_section();
    }


    /**
     * Add widget controls: Content -> Button Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_button_content() {
        $this->start_controls_section('section_content_button_content', [
            'label' => esc_html__('Button content', 'uper-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'button_text',
            [
                'label' => esc_html__( 'Button text', 'uper-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Up', 'uper-elementor' ),
                'placeholder' => esc_html__( 'Type button text here', 'uper-elementor' ),
            ]
        );

        $this->add_control(
            'button_text_alignment',
            [
                'label' => esc_html__( 'Alignment', 'uper-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'uper-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'uper-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'uper-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-uper-elementor-button' => 'justify-content: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'button_icon',
            [
                'label' => esc_html__( 'Button icon', 'uper-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-long-arrow-alt-up',
                    'library' => 'solid',
                ],
            ]
        );

        $this->add_control(
            'button_icon_position',
            [
                'label' => esc_html__( 'Icon position', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'before',
                'options' => [
                    'before' => esc_html__( 'Before', 'uper-elementor' ),
                    'after' => esc_html__( 'After', 'uper-elementor' ),
                    'above-left' => esc_html__( 'Above left', 'uper-elementor' ),
                    'above-center' => esc_html__( 'Above center', 'uper-elementor' ),
                    'above-right' => esc_html__( 'Above right', 'uper-elementor' ),
                    'under-left' => esc_html__( 'Under left', 'uper-elementor' ),
                    'under-center' => esc_html__( 'Under center', 'uper-elementor' ),
                    'under-right' => esc_html__( 'Under right', 'uper-elementor' )
                ],
            ]
        );

        $this->add_control(
            'button_content_appearance_animation',
            [
                'label' => esc_html__( 'Content appearance animation', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'label_block' => true,
                'default' => 'none',
                'options' => [
                    'grow' => esc_html__( 'Grow', 'uper-elementor' ),
                    'shrink' => esc_html__( 'Shrink', 'uper-elementor' ),
                    'slide-up' => esc_html__( 'Slide up', 'uper-elementor' ),
                    'slide-down' => esc_html__( 'Slide down', 'uper-elementor' ),
                    'fade' => esc_html__( 'Fade', 'uper-elementor' ),
                    'none' => esc_html__( 'None', 'uper-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'button_content_appearance_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'uper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'uper-elementor' ),
                    'ease-in' => esc_html__( 'Ease-in', 'uper-elementor' ),
                    'ease-out' => esc_html__( 'Ease-out', 'uper-elementor' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'uper-elementor' ),
                    'linear' => esc_html__( 'Linear', 'uper-elementor' ),
                ],
                'condition' => [
                    'button_content_appearance_animation!' => 'none',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-text-wrapper' => 'animation-timing-function: {{VALUE}} !important'
                ],
            ]
        );

        $this->add_control(
            'button_content_appearance_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-text-wrapper' => 'animation-duration: {{SIZE}}{{UNIT}} !important'
                ],
                'condition' => [
                    'button_content_appearance_animation!' => 'none',
                ],
            ]
        );

        $this->add_control(
            'button_content_appearance_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-text-wrapper' => 'animation-delay: {{SIZE}}{{UNIT}} !important'
                ],
                'condition' => [
                    'button_content_appearance_animation!' => 'none',
                ],
            ]
        );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_button() {

        $this->start_controls_section( 'section_style_button', [
            'label' => esc_html__( 'Button', 'uper-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'button_margin',
            [
                'label' => esc_html__( 'Margin', 'uper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );

        $this->add_responsive_control(
            'button_padding',
            [
                'label' => esc_html__( 'Padding', 'uper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'separator' => 'after',
                'default' => [
                    'top' => '10',
                    'right' => '25',
                    'bottom' => '10',
                    'left' => '25',
                    'isLinked' => false,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );

        $this->add_responsive_control(
            'icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'uper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button-icon' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'icon_size',
            [
                'label' => esc_html__( 'Icon size', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button-icon' => 'font-size: {{SIZE}}{{UNIT}}',
                    '{{WRAPPER}} .mdp-uper-elementor-button-icon svg' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
                'separator' => 'after'
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'button_title_typography',
                'label' => esc_html__( 'Typography', 'uper-elementor' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-text-wrapper',
            ]
        );

        $this->start_controls_tabs('control_style_tabs');

        $this->start_controls_tab( 'normal_style_tab', ['label' => esc_html__( 'NORMAL', 'uper-elementor' )] );

        $this->add_control(
            'button_text_color',
            [
                'label' => esc_html__( 'Text color', 'uper-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button-title' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'icon_color',
            [
                'label' => esc_html__( 'Icon color', 'uper-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button-icon' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'button_background',
                'label' => esc_html__( 'Background type', 'uper-elementor' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-button',
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'border_button_normal',
                'label' => esc_html__( 'Border Type', 'uper-elementor' ),
                'separator' => 'before',
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-button',
            ]
        );

        $this->add_responsive_control(
            'button_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'uper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'button_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'uper-elementor' ),
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-button',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab( 'hover_style_tab', ['label' => esc_html__( 'HOVER', 'uper-elementor' )] );

        $this->add_control(
            'button_text_color_hover',
            [
                'label' => esc_html__( 'Text color', 'uper-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button:hover .mdp-uper-elementor-button-title' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'icon_color_hover',
            [
                'label' => esc_html__( 'Icon color', 'uper-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button:hover .mdp-uper-elementor-button-icon' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'button_background_hover',
                'label' => esc_html__( 'Background type', 'uper-elementor' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-button:hover',
            ]
        );


        $this->add_control(
            'button_hover_transition',
            [
                'label' => esc_html__( 'Hover transition', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button' => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'border_button_hover',
                'label' => esc_html__( 'Border Type', 'uper-elementor' ),
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-button:hover',
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'button_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'uper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-button:hover' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'button_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'uper-elementor' ),
                'selector' => '{{WRAPPER}} .mdp-uper-elementor-button:hover',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Scroll Indicator.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_scroll_indicator()
    {
        $this->start_controls_section('section_style_scroll_indicator', [
            'label' => esc_html__('Scroll indicator', 'uper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                    'scroll_indicator' => 'yes',
                    'fixed' => 'yes'
            ]
        ] );

        $this->add_control(
            'indicator_color',
            [
                'label' => esc_html__( 'Indicator color', 'uper-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-scroll-indicator-circle' => 'stroke: {{VALUE}}'
                ],
                'condition' => [
                    'scroll_indicator_type' => 'circle'
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'indicator_background_color',
                'label' => esc_html__( 'Indicator color', 'uper-elementor' ),
                'types' => [ 'classic', 'gradient' ],
                'exclude' => ['image'],
                'selector' => '{{WRAPPER}} .mdp-upper-elementor-background-indicator',
                'condition' => [
                    'scroll_indicator_type' => 'background'
                ]
            ]
        );

        $this->add_responsive_control(
            'indicator_line_width',
            [
                'label' => esc_html__( 'Indicator line width', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 50,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-uper-elementor-scroll-indicator-circle' => 'stroke-width: {{SIZE}}{{UNIT}};',
                ],
                'condition' => [
                    'scroll_indicator_type' => 'circle'
                ]
            ]
        );

        $this->add_responsive_control(
            'indicator_radius',
            [
                'label' => esc_html__( 'Radius', 'uper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'size' => 30,
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'size' => 25,
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'size' => 25,
                    'unit' => 'px',
                ],
                'condition' => [
                    'scroll_indicator_type' => 'circle'
                ]
            ]
        );

        $this->add_control(
            'rounded_stroke_linecap',
            [
                'label' => esc_html__( 'Rounded stoke linecap', 'uper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'On', 'uper-elementor' ),
                'label_off' => esc_html__( 'Off', 'uper-elementor' ),
                'return_value' => 'yes',
                'separator' => 'after',
                'default' => 'yes',
                'condition' => [
                    'scroll_indicator_type' => 'circle'
                ]
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        $is_enabled_scroll_indicator = $settings['scroll_indicator'] === 'yes' && $settings['fixed'] === 'yes';
        ?>
        <!-- Start Uper for Elementor WordPress Plugin -->
        <div class="mdp-uper-elementor-box"
         <?php if ( $settings['start_displaying_button_position'] === 'pixels' || $settings['start_displaying_button_position'] === 'percentage' ): ?>
            data-offset="<?php $settings['start_displaying_button_position'] === 'pixels' ? esc_attr_e( $settings['start_displaying_button_position_pixels'] ) : esc_attr_e( $settings['start_displaying_button_position_percentage'] ); ?>"
         <?php endif; ?>
            data-offset-type="<?php esc_attr_e( $settings['start_displaying_button_position'] ); ?>"
         <?php if ( $settings['start_displaying_button_position'] === 'custom-element' ): ?>
            data-custom-element-id="<?php esc_attr_e( $settings['start_displaying_button_position_element'] ); ?>"
         <?php endif; ?>
         <?php if( $settings['auto_hide'] === 'yes' ): ?>
            data-autohide="<?php esc_attr_e( $settings['auto_hide'] ); ?>"
            data-autohide-seconds="<?php esc_attr_e( $settings['auto_hide_after'] ); ?>"
         <?php endif; ?>
             data-button-hover-animation="<?php esc_attr_e( $settings['button_hover_animation'] ); ?>"
             data-button-appearance-animation="<?php esc_attr_e( $settings['button_appearance_animation'] ); ?>"
             data-button-content-appearance-animation="<?php esc_attr_e( $settings['button_content_appearance_animation'] ); ?>"
             data-scroll-to="<?php esc_attr_e( $settings['scroll_to'] ); ?>"
             <?php if ( $settings['scroll_to'] === 'anchor' ): ?>
                data-anchor-id="<?php esc_attr_e( $settings['anchor_id'] ); ?>"
             <?php endif; ?>
             data-scroll-speed="<?php esc_attr_e( $settings['scroll_speed']['size'] ); ?>"
             data-scroll-easing="<?php esc_attr_e( $settings['scroll_easing'] ); ?>"
             data-is-fixed="<?php esc_attr_e( $settings['fixed'] ); ?>"
             data-scroll-indicator="<?php esc_attr_e( $settings['scroll_indicator'] ); ?>"
             data-scroll-indicator-type="<?php esc_attr_e( $settings['scroll_indicator_type'] ); ?>"
            <?php if ( array_key_exists( 'widescreen', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['indicator_radius_widescreen'] ) ): ?> data-radius-widescreen="<?php esc_attr_e( $settings['indicator_radius_widescreen']['size'] ); ?>" <?php endif; ?>
            <?php if ( array_key_exists( 'laptop', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['indicator_radius_laptop']['size'] ) ): ?> data-radius-laptop="<?php esc_attr_e( $settings['indicator_radius_laptop']['size'] ); ?>" <?php endif; ?>
            <?php if ( array_key_exists( 'tablet_extra', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['indicator_radius_tablet_extra'] ) ): ?> data-radius-tablet-extra="<?php esc_attr_e( $settings['indicator_radius_tablet_extra']['size']); ?>" <?php endif; ?>
            <?php if ( array_key_exists( 'mobile_extra', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['indicator_radius_mobile_extra'] ) ): ?> data-radius-mobile-extra="<?php esc_attr_e( $settings['indicator_radius_mobile_extra']['size'] ); ?>" <?php endif; ?>
             <?php if ( $settings['scroll_indicator'] === 'yes' && $settings['fixed'] === 'yes' ): ?> data-radius="<?php esc_attr_e( $settings['indicator_radius']['size'] ); ?>" <?php endif; ?>
             <?php if ( isset( $settings['indicator_radius_tablet'] ) ): ?> data-radius-tablet="<?php esc_attr_e( $settings['indicator_radius_tablet']['size'] ); ?>" <?php endif; ?>
             <?php if ( isset( $settings['indicator_radius_mobile'] ) ): ?> data-radius-mobile="<?php esc_attr_e( $settings['indicator_radius_mobile']['size'] ); ?>" <?php endif; ?>
        >
                <div class="mdp-uper-elementor-button <?php if( $settings['fixed'] === 'yes' ):  ?>
                mdp-uper-elementor-button-fixed mdp-uper-elementor-button-fixed-position-<?php esc_attr_e( $settings['fixed_button_position'] ); ?>  <?php endif; ?>" <?php if ( !is_admin() ): ?> style="opacity: 0" <?php endif; ?> >
                    <?php if ( $is_enabled_scroll_indicator && $settings['scroll_indicator_type'] === 'circle' ): ?>
                        <svg class="mdp-uper-elementor-scroll-indicator-svg">
                            <g>
                                <circle class="mdp-uper-elementor-scroll-indicator-circle"
                                        r="<?php esc_attr_e( $settings['indicator_radius']['size'] ); ?>"
                                        cx="<?php esc_attr_e( $settings['indicator_radius']['size'] ); ?>"
                                        cy="<?php esc_attr_e( $settings['indicator_radius']['size'] ); ?>"
                                    <?php if ( $settings['rounded_stroke_linecap'] === 'yes' ): ?> stroke-linecap="round" <?php endif; ?>
                                >
                            </g>
                        </svg>
                    <?php elseif ( $is_enabled_scroll_indicator && $settings['scroll_indicator_type'] === 'background' ): ?>
                        <div class="mdp-upper-elementor-background-indicator"></div>
                    <?php endif; ?>
                 <div class="mdp-uper-elementor-text-wrapper mdp-uper-elementor-icon-position-<?php esc_attr_e( $settings['button_icon_position'] ); ?>">
                     <div class="mdp-uper-elementor-button-icon">
                         <?php Icons_Manager::render_icon( $settings['button_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                     </div>
                     <div class="mdp-uper-elementor-button-title">
                         <?php esc_attr_e( $settings['button_text'] ); ?>
                     </div>
                 </div>
            </div>
        </div>
        <?php if( $settings['fixed'] === 'yes' ): ?>
            <style>
                .elementor-element-<?php esc_html_e( $this->get_id() ); ?> .elementor-widget-empty-icon {
                    padding: 0 !important;
                }

                .elementor-widget-mdp-uper-elementor {
                    animation: 0;
                }

            </style>
        <?php endif; ?>
        <!-- End Uper for Elementor WordPress Plugin -->
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>
            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    uperReady<?php esc_attr_e( $widget_hash ); ?>( mdpUper.addUper.bind( mdpUper ) );
                } catch ( msg ) {
                    const uperReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    uperReady<?php esc_attr_e( $widget_hash ); ?>( mdpUper.addUper.bind( mdpUper ) );
                }
            </script>
	    <?php
        }  else { ?>
            <script>
                window.addEventListener( 'load', () => {
                    document.querySelectorAll( '.elementor-element-<?php esc_attr_e( $this->get_id() ); ?> .mdp-uper-elementor-button' ).forEach( dropdown => {
                        dropdown.style.opacity = 1;
                    } );
                } );
            </script>
            <?php
        }
    }


    /**
     * JS Render.
     *
     * @access protected
     *
     * @return void
     **/
    function content_template()
    {
        ?>
        <# const isEnabledScrollIndicator = settings.scroll_indicator === 'yes' && settings.fixed === 'yes'; #>
        <!-- Start Uper for Elementor WordPress Plugin -->
        <div class="mdp-uper-elementor-box"
            <# if( settings.start_displaying_button_position === 'pixels' || settings.start_displaying_button_position === 'percentage' ) { #>
                data-offset="<# if ( settings.start_displaying_button_position === 'pixels' ) { #>
                        {{{ settings.start_displaying_button_position_pixels }}}
                <# } else { #>
                    {{{ settings.start_displaying_button_position_percentage }}}
                <# } #>"
            <#}#>
             data-offset-type="{{{ settings.start_displaying_button_position }}}"
            <# if ( settings.start_displaying_button_position === 'custom-element' ) { #>
                data-custom-element-id="{{{ settings.start_displaying_button_position_element }}}"
            <# } #>
            <# if ( settings.auto_hide === 'yes' ) { #>
                data-autohide="{{{ settings.auto_hide }}}"
                data-autohide-seconds="{{{ settings.auto_hide_after }}}"
            <# } #>
             data-button-hover-animation="{{{ settings.button_hover_animation }}}"
             data-button-appearance-animation="{{{ settings.button_appearance_animation }}}"
             data-button-content-appearance-animation={{{ settings.button_content_appearance_animation }}}
             data-scroll-to="{{{ settings.scroll_to }}}"
            <# if ( settings.scroll_to === 'anchor' ) { #>
                data-anchor-id="{{{ settings.anchor_id }}}"
            <# } #>
             data-scroll-speed="{{{ settings.scroll_speed.size }}}"
             data-scroll-easing="{{{ settings.scroll_easing }}}"
             data-is-fixed="{{{ settings.fixed }}}"
             data-scroll-indicator="{{{ settings.scroll_indicator }}}"
             data-scroll-indicator-type="{{{ settings.scroll_indicator_type }}}"
             <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' ) && settings.indicator_radius_widescreen ) { #> data-radius-widescreen="{{{ settings.indicator_radius_widescreen.size }}}" <# } #>
             <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'laptop' ) && settings.indicator_radius_laptop ) { #> data-radius-laptop="{{{ settings.indicator_radius_laptop.size }}}" <# } #>
             <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'tablet_extra' ) && settings.indicator_radius_tablet_extra ) { #> data-radius-tablet-extra="{{{ settings.indicator_radius_tablet_extra.size }}}" <# } #>
             <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'mobile_extra' ) && settings.indicator_radius_mobile_extra ) { #> data-radius-mobile-extra="{{{ settings.indicator_radius_mobile_extra.size }}}" <# } #>
             data-radius="{{{ settings.indicator_radius.size }}}"
             <# if ( settings.indicator_radius_tablet ) { #> data-radius-tablet="{{{ settings.indicator_radius_tablet.size }}}" <# } #>
             <# if ( settings.indicator_radius_mobile ) { #> data-radius-mobile="{{{ settings.indicator_radius_mobile.size }}}" <# } #>
            >
            <div class="mdp-uper-elementor-button <# if ( settings.fixed === 'yes' ) { #>
                mdp-uper-elementor-button-fixed mdp-uper-elementor-button-fixed-position-{{{ settings.fixed_button_position }}} <# } #>">
                <# if ( isEnabledScrollIndicator && settings.scroll_indicator_type === 'circle' ) { #>
                    <svg class="mdp-uper-elementor-scroll-indicator-svg">
                        <g>
                            <circle class="mdp-uper-elementor-scroll-indicator-circle"
                                    r="{{{ settings.indicator_radius.size }}}"
                                    cx="{{{ settings.indicator_radius.size }}}"
                                    cy="{{{ settings.indicator_radius.size }}}"
                                <# if ( settings.rounded_stroke_linecap === 'yes' ) { #> stroke-linecap="round" <# } #>
                            >
                        </g>
                    </svg>
                <# } else if ( isEnabledScrollIndicator && settings.scroll_indicator_type === 'background' ) { #>
                    <div class="mdp-upper-elementor-background-indicator"></div>
                <# } #>
                <div class="mdp-uper-elementor-text-wrapper mdp-uper-elementor-icon-position-{{{ settings.button_icon_position }}}">
                    <div class="mdp-uper-elementor-button-icon">
                        <# var iconHTML = elementor.helpers.renderIcon( view, settings.button_icon, { 'aria-hidden': true }, 'i' , 'object' ); #>
                        <# if ( !iconHTML.rendered ) { #>
                            <i class="{{{ settings.button_icon.value }}}" ></i>
                        <# } else { #>
                            {{{ iconHTML.value }}}
                        <# } #>
                    </div>
                    <div class="mdp-uper-elementor-button-title">
                        {{{ settings.button_text }}}
                    </div>
                </div>
            </div>
        </div>
        <# if ( settings.fixed === 'yes' ) { #>
        <style>
            .elementor-element-<?php esc_html_e( $this->get_id() ); ?> .elementor-widget-empty-icon {
                padding: 0 !important;
            }

            .elementor-widget-mdp-uper-elementor {
                animation: 0;
            }
        </style>
        <# } #>
        <!-- End Uper for Elementor WordPress Plugin -->
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>
            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    uperReady<?php esc_attr_e( $widget_hash ); ?>( mdpUper.addUper.bind( mdpUper ) );
                } catch ( msg ) {
                    const uperReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    uperReady<?php esc_attr_e( $widget_hash ); ?>( mdpUper.addUper.bind( mdpUper ) );
                }
            </script>
            <?php
        }
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/uper';

    }

}
